#include "Normalizer.h"

//FSM Normalizer::fsm;
//vector <NormRule> Normalizer::nRules;

Normalizer::Normalizer()
{
}

Normalizer::~Normalizer()
{
	Clear();
}

void Normalizer::Clear ()
{
	if (fsm.isDefined())
		fsm.deAllocFSM();

	nRules.clear();
}

bool Normalizer::LoadNormRules (string filename)
{
	if (!fsm.isDefined()) // init FSM, if not already initialized
	{
		if(!ReadPatterns (filename.c_str()))
			return false;
		vector <wstring> pList;
		for (int i = 0; i < nRules.size(); i++)
			pList.push_back (nRules[i].pattern);
		fsm.allocFSM (pList);
	}

	return true;
}

wstring Normalizer::NormalizeString(wstring str)
{
	wstring resStr;

	// convert "letter + diacs + do_cheshmi_he" into "letter + do_cheshmi_he + diacs"
	resStr = DiacriticHehDoChasmee(str); 
	// apply norm rules on given str
	ReplaceMatches (resStr);

	return resStr;
}

// convert "letter + diacs + do_cheshmi_he" into "letter + do_cheshmi_he + diacs"
wstring Normalizer::DiacriticHehDoChasmee(wstring str) 
{	
	wstring resStr;
	wchar_t curCh, prevCh = 0;
	int i;
	for (i = 0; i < str.length(); i++)
	{
		curCh = str[i];

	
		// convert "letter + diacs + do_cheshmi_he" into "letter + do_cheshmi_he + diacs"
		if (curCh == __DO_CHESHMI_HAH && i > 0 && IsDiac(resStr[i-1]))
		{
			int j = i-1;
			while (j >= 0 && IsDiac(str[j]))
				j--;
			resStr.insert (j+1, 1, __DO_CHESHMI_HAH);
			continue;
		}

		// append in resultant string
		resStr += curCh;
		prevCh = curCh;
	}
	return resStr;
}

bool Normalizer::ReadPatterns (const char* filename)
{
	FileReader fr;
	try
	{
		if(!fr.Open (filename, FileReader::UNI))
			return false;

		wstring tempStr;
		NormRule nr;
		wchar_t wc;
		fr.Read (&wc, 1);
		while (!fr.EoF())
		{
			if (wc == 0x003A) // char (:)
			{
				nr.pattern = tempStr;	// pattern string finished
				tempStr.erase();
			}
			else if (wc == 0x000D)	// char (\r)
			{
				fr.Read (&wc, 1); // consume char (\n)
				/*if (wc != 0x000A) // if mistakenly consume char other than (\n)
					fr.UngetUnicodeCharacters (1);*/
				nr.replace = tempStr;
				nRules.push_back (nr);
				
				nr.replace.erase();
				tempStr.erase();


			}
			
			else	// some other char
				tempStr += wc;

			fr.Read (&wc, 1);
		}
	}
	catch (IOException& ex)
	{
		// set params telling "where" actaully it caused exception
		ex.SetFileName (__FILE__);
		ex.SetLineNum (__LINE__);
		ex.SetExceptionMsg (ex.GetExceptionMsg() 
				+ " File reading problem occurred in Normalization Rules file.");

		return false;
		throw; // re-throw the same exception object
	}

	return true;
}

void Normalizer::ReplaceMatches (wstring& text)
{
	POccurrence po;
	po = fsm.find (text, 0);
	if (po.location == 0)	// some pattern matched at start
	{
		text.replace(po.location, nRules[po.patternID].pattern.length(), 
				nRules[po.patternID].replace);
		po = fsm.find (text, po.location + nRules[po.patternID].replace.length());
	}

	// find until some pattern matches in middle of the text string
	while (po.location != -1 
			&& po.location + nRules[po.patternID].pattern.length() < text.length())
	{
		text.replace(po.location, nRules[po.patternID].pattern.length(), 
				nRules[po.patternID].replace);
		po = fsm.find (text, po.location + nRules[po.patternID].replace.length());
	}

	// some pattern matched at end of text string
	if (po.location != -1 
			&& po.location + nRules[po.patternID].pattern.length() == text.length())
		text.replace(po.location, nRules[po.patternID].pattern.length(), 
				nRules[po.patternID].replace);

	return;
}

// tests whether or not the given character is diacritic
bool Normalizer::IsDiac (wchar_t wch)
{
	if (wch == __zebr || wch == __zer || wch == __pesh || wch == __jezm ||
			wch == __kzebr_above || wch == __do_zebr || wch == __shd || wch == __med ||
			wch == __kzebr_below || wch == __hamza || wch == __upesh || wch == __noon ||
			wch == __sallah || wch == __alehe || wch == __rehmat || wch == __razi || wch == __takhalus || wch == __toe)
		return true;
	else
		return false;
}


